import { useRef, useMemo, useCallback, useEffect } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var allEventTypes = {
    "keydown": "onKeyDown",
    "keyup": "onKeyUp",
    "keypress": "onKeyPress",
};
var defaultProps = {
    eventTypes: ["keydown"]
};
var defaultState = function () { return ({
    listeners: new Map(),
    options: new Map()
}); };
var isKeyEvent = function (key) {
    return key.altKey !== undefined;
};
var convert = function (key) {
    if (isKeyEvent(key)) {
        return key.key + "," + key.altKey + "," + key.ctrlKey + "," + key.shiftKey + "," + key.metaKey;
    }
    return key.key + "," + (key.alt || false) + "," + (key.ctrl || false) + "," + (key.shift || false) + "," + (key.meta || false);
};
var listenerKeys = function (option) {
    if (Array.isArray(option.key)) {
        return option.key.map(function (key) { return convert({ key: key }); });
    }
    if (typeof option.key === "string") {
        return [convert({ key: option.key })];
    }
    var keyProp = option.key;
    if (Array.isArray(keyProp.key)) {
        return keyProp.key.map(function (key) { return convert(__assign(__assign({}, keyProp), { key: key })); });
    }
    return [convert(__assign(__assign({}, keyProp), { key: keyProp.key }))];
};
var createState = function (props) {
    if (Array.isArray(props.listeners)) {
        return props.listeners.flatMap(function (option) {
            return listenerKeys(option).map(function (key) { return (__assign(__assign({}, option), { key: key })); });
        })
            .reduce(addToState, defaultState());
    }
    return Object.entries(props.listeners)
        .map(function (entry) { return ({
        key: convert({ key: entry[0] }),
        callback: typeof entry[1] === "function" ? entry[1] : entry[1].callback,
        options: typeof entry[1] === "function" ? {} : entry[1].options || {}
    }); })
        .reduce(addToState, defaultState());
};
var addToState = function (state, entry) {
    var _a, _b;
    var key = entry.key;
    if (!state.listeners.has(key)) {
        state.listeners.set(key, []);
        state.options.set(key, []);
    }
    (_a = state.listeners.get(key)) === null || _a === void 0 ? void 0 : _a.push(entry.callback);
    (_b = state.options.get(key)) === null || _b === void 0 ? void 0 : _b.push(entry.options || {});
    return state;
};
function useKeyboard(props) {
    var shouldListen = props.shouldListen;
    var active = useRef(false);
    var state = useRef(createState(props));
    var options = useMemo(function () {
        return __assign(__assign({}, defaultProps), props);
    }, [props]);
    var eventListener = useCallback(function (event) {
        var _a;
        var key = convert(event);
        if (state.current.listeners.has(key)) {
            if (options.preventDefault)
                event.preventDefault();
            (_a = state.current.listeners.get(key)) === null || _a === void 0 ? void 0 : _a.forEach(function (listener, index) {
                var _a;
                if ((_a = state.current.options.get(key)) === null || _a === void 0 ? void 0 : _a[index].preventDefault)
                    event.preventDefault();
                listener(event);
            });
        }
    }, [options.preventDefault]);
    var register = useCallback(function (operation) {
        var _a;
        if ((_a = options.elementRef) === null || _a === void 0 ? void 0 : _a.current) {
            options.eventTypes.forEach(function (eventType) { var _a; return (_a = options.elementRef) === null || _a === void 0 ? void 0 : _a.current[operation](eventType, eventListener); });
        }
        else {
            options.eventTypes.forEach(function (eventType) { return document[operation](eventType, eventListener); });
        }
    }, [options.eventTypes, eventListener, options.elementRef]);
    var listenerHandlers = useMemo(function () {
        return {
            addKeyboardListener: function () {
                if (active.current)
                    return;
                register("addEventListener");
                active.current = true;
            },
            removeKeyboardListener: function () {
                if (!active.current)
                    return;
                register("removeEventListener");
                active.current = false;
            },
        };
    }, [register]);
    var keyboardHandlers = useMemo(function () {
        return options.eventTypes.map(function (ev) { return allEventTypes[ev]; })
            .reduce(function (prev, curr) {
            var _a;
            return (__assign(__assign({}, prev), (_a = {}, _a[curr] = eventListener, _a)));
        }, {});
    }, [options.eventTypes, eventListener]);
    useEffect(function () {
        if (shouldListen === undefined)
            return;
        if ((typeof shouldListen === "function" && shouldListen()) || shouldListen === true) {
            listenerHandlers.addKeyboardListener();
        }
        else {
            listenerHandlers.removeKeyboardListener();
        }
        return function () {
            listenerHandlers.removeKeyboardListener();
        };
    }, [shouldListen, listenerHandlers]);
    return __assign(__assign({}, listenerHandlers), { eventListener: eventListener, keyboardHandlers: keyboardHandlers });
}

export { useKeyboard };
//# sourceMappingURL=index.es.js.map
