import React from "react";
export declare type KeyEvent = KeyboardEvent | React.KeyboardEvent;
export declare type KeyEventMap = {
    [type in "keydown" | "keyup" | "keypress"]: keyof KeyboardHandlers;
};
export declare type KeyEventListener = (event: KeyEvent) => void;
export declare type KeyConfig = Partial<Modifiers> & {
    key: string | Array<string>;
};
export declare type Modifiers = {
    alt: boolean;
    ctrl: boolean;
    shift: boolean;
    meta: boolean;
};
export declare type KeyboardProps = KeyboardListenAndElementRef & {
    listeners: Array<ListenerConfig> | SimpleListenerConfig;
} & Partial<KeyboardEventTypes>;
export declare type KeyboardListenAndElementRef = ListenerConfigOptions & {
    shouldListen?: boolean | (() => boolean);
    elementRef?: React.MutableRefObject<HTMLElement>;
};
export declare type KeyboardOptions = KeyboardListenAndElementRef & KeyboardEventTypes;
export declare type KeyboardEventTypes = {
    eventTypes: Array<keyof KeyEventMap>;
};
export declare type SimpleListenerConfig = {
    [key: string]: KeyEventListener | ListenerCallbackAndOptions;
};
export declare type ListenerConfig = ListenerCallbackAndOptions & {
    key: string | Array<string> | KeyConfig;
};
export declare type ListenerCallbackAndOptions = {
    callback: KeyEventListener;
    options?: ListenerConfigOptions;
};
export declare type ListenerConfigOptions = {
    preventDefault?: boolean;
};
export declare type KeyboardHandlers = {
    onKeyDown?(event: React.KeyboardEvent): void;
    onKeyPress?(event: React.KeyboardEvent): void;
    onKeyUp?(event: React.KeyboardEvent): void;
};
export declare type KeyboardListenerHandlers = {
    addKeyboardListener(): void;
    removeKeyboardListener(): void;
};
export declare type KeyboardResult = KeyboardListenerHandlers & {
    eventListener: KeyEventListener;
    keyboardHandlers: KeyboardHandlers;
};
export declare type KeyboardState = {
    listeners: MultiValueMap<KeyEventListener>;
    options: MultiValueMap<ListenerConfigOptions>;
};
export declare type MultiValueMap<T> = Map<string, Array<T>>;
export declare type EventListenerTarget = {
    addEventListener(event: keyof KeyEventMap, callback: KeyEventListener): void;
    removeEventListener(event: keyof KeyEventMap, callback: KeyEventListener): void;
};
